/**
 * 
 */
package com.rjconsultores.ventaboletos.relatorios.impl;

import java.sql.Connection;
import java.sql.ResultSet;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import com.rjconsultores.ventaboletos.relatorios.utilitarios.ArrayDataSource;
import com.rjconsultores.ventaboletos.utilerias.DateUtil;
import com.rjconsultores.ventaboletos.web.utilerias.NamedParameterStatement;

public class RelatorioFormaPagamentoAgenciaNovo extends RelatorioDemandas {
	public RelatorioFormaPagamentoAgenciaNovo(Map<String, Object> parametros, Connection conexao) throws Exception {
		super(parametros, conexao);
		this.setCustomDataSource(new ArrayDataSource(this) {
			@Override
			public void initDados() throws Exception {

				Connection conexao = this.relatorio.getConexao();
				Map<String, Object> parametros = this.relatorio.getParametros();

				Date dataInicial = (Date) parametros.get("DATA_INICIAL");
				Date dataFinal = (Date) parametros.get("DATA_FINAL");				
				Integer empresa = (Integer) parametros.get("EMPRESA");
				Integer agencia = (Integer) parametros.get("AGENCIA");				
				Short formaPagoId = (Short) parametros.get("FORMA_PAGO");	        
		        
		        StringBuilder sql = new StringBuilder();
		        sql.append(" SELECT " );
		        sql.append("     c.fechorventa                      AS data, " );
		        sql.append("     c.numfoliosistema                  AS bilhete, " );
		        sql.append("     pv.nombpuntoventa                  AS agencia, " );
		        sql.append("     p_origen.CVEPARADA                 AS origem , " );
		        sql.append("     p_destino.CVEPARADA                AS destino, " );
		        sql.append("     fp.descpago                        AS forma_pagamento, " );
		        sql.append("     coalesce(c.preciopagado, 0)        AS tarifa, " );
		        sql.append("     coalesce(c.importepedagio, 0)      AS pedagio, " );
		        sql.append("     coalesce(c.importeseguro, 0)       AS seguro, " );
		        sql.append("     coalesce(c.importetaxaembarque, 0) AS taxa, " );
		        sql.append("     coalesce(cfp.importe, 0)           AS valor,  " );
		        sql.append("     coalesce(seg.valor, 0)             AS seg_opcional " );
		        sql.append(" FROM " );
		        sql.append("     caja c " );
		        sql.append("     JOIN caja_formapago cfp ON c.caja_id = cfp.caja_id " );
		        sql.append("     JOIN forma_pago     fp ON cfp.formapago_id = fp.formapago_id " );
		        sql.append("     JOIN punto_venta    pv ON pv.puntoventa_id = c.puntoventa_id " );
		        sql.append("     JOIN marca          m  ON m.marca_id = c.marca_id AND m.activo = 1 " );
		        sql.append("     JOIN parada p_origen   ON p_origen.parada_id = c.origen_id  " );
		        sql.append("     JOIN parada p_destino  ON p_destino.parada_id = c.destino_id  " );
		        sql.append("     LEFT JOIN segpolv seg ON c.transacao_id = seg.boleto_id " );
		        sql.append(" WHERE " );
		        sql.append("     c.activo = 1 " );
		        sql.append("     AND c.indreimpresion = 0 " );
	        	sql.append("     AND c.fechorventa >= :dataInicial " );
	        	sql.append("     AND c.fechorventa <= :dataFinal " );		        
		        
		        if (empresa != null) {
		        	sql.append(" AND c.empresa_id  = :empresaId ");
		        }
		        
		        if (agencia != null) {
		        	sql.append(" AND c.puntoventa_id = :agenciaId ");
		        }
		        
		        if (formaPagoId != null) {
		        	sql.append(" AND cfp.FORMAPAGO_ID = :formaPagoId ");
		        }
		        
		        sql.append("  ORDER BY data, agencia, bilhete " );

				NamedParameterStatement stmt = new NamedParameterStatement(conexao, sql.toString());
								
				stmt.setDate("dataInicial", DateUtil.inicioFechaSql(dataInicial));
				stmt.setDate("dataFinal", DateUtil.fimFechaSql(dataFinal));
				
				if (empresa != null) {
					stmt.setInt("empresaId", empresa);					
				}
				
				if (agencia != null) {
					stmt.setInt("agenciaId", agencia);					
				}
				
				if (formaPagoId != null ) {
					stmt.setInt("formapagoId", formaPagoId);
				}

				ResultSet rset = stmt.executeQuery();

				while (rset.next()) {
					Map<String, Object> dataResult = new HashMap<String, Object>();
				        
					dataResult.put("DATA", rset.getDate("data"));
					dataResult.put("BILHETE", rset.getString("bilhete"));
					dataResult.put("ORIGEM", rset.getString("ORIGEM"));
					dataResult.put("DESTINO", rset.getString("DESTINO"));
					dataResult.put("AGENCIA", rset.getString("agencia"));
					dataResult.put("FORMA_PAGAMENTO", rset.getString("forma_pagamento"));
					dataResult.put("TARIFA", rset.getBigDecimal("tarifa"));
					dataResult.put("PEDAGIO", rset.getBigDecimal("PEDAGIO"));
					dataResult.put("SEGURO", rset.getBigDecimal("SEGURO"));
					dataResult.put("SEG_OPCIONAL", rset.getBigDecimal("seg_opcional"));
					dataResult.put("TAXA", rset.getBigDecimal("taxa"));
					dataResult.put("VALOR", rset.getBigDecimal("valor"));

					this.dados.add(dataResult);
				}

				this.resultSet = rset;
			}
		});
	}
}
