package com.rjconsultores.ventaboletos.web.gui.controladores.catalogos;

import java.util.HashMap;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Paging;
import org.zkoss.zul.Textbox;

import com.rjconsultores.ventaboletos.entidad.ContaMD;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderContaMD;

@Scope("prototype")
@Controller("busquedaContasMDController")
public class BusquedaContasMDController extends MyGenericForwardComposer {
	private static final long serialVersionUID = 1L;

	@Autowired
	private transient PagedListWrapper<ContaMD> plwConta;
	private MyListbox contaList;
	private Paging pagingConta;
	private Textbox txtNombre;

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		super.doAfterCompose(comp);

		contaList.setItemRenderer(new RenderContaMD());
		contaList.addEventListener("onDoubleClick", new EventListener() {
			@Override
			public void onEvent(Event event) throws Exception {
				ContaMD tc = (ContaMD) contaList.getSelected();
				verContaMD(tc);
			}
		});

		refreshLista();

		txtNombre.focus();
	}

	public Paging getPagingTipoInformativo() {
		return pagingConta;
	}

	public void setPagingTipoInformativo(Paging pagingTipoInformativo) {
		this.pagingConta = pagingTipoInformativo;
	}

	public PagedListWrapper<ContaMD> getPlwTipoInformativo() {
		return plwConta;
	}

	public void setPlwTipoInformativo(PagedListWrapper<ContaMD> plwConta) {
		this.plwConta = plwConta;
	}

	public MyListbox getTipoInformativoList() {
		return contaList;
	}

	public void setTipoInformativoList(MyListbox tipoInformativoList) {
		this.contaList = tipoInformativoList;
	}

	public Textbox getTxtNombre() {
		return txtNombre;
	}

	public void setTxtNombre(Textbox txtNombre) {
		this.txtNombre = txtNombre;
	}

	public void onClick$btnPesquisa(Event ev) throws InterruptedException {
		refreshLista();
	}

	public void onClick$btnRefresh(Event ev) {
		refreshLista();
	}

	public void onClick$btnNovo(Event ev) {
		verContaMD(new ContaMD());
	}

	private void verContaMD(ContaMD contaMD) {
		if (contaMD == null) {
			return;
		}

		Map<String, Object> args = new HashMap<String, Object>();
		args.put("conta", contaMD);
		args.put("contaList", contaList);

		openWindow("/gui/catalogos/editarContaMD.zul",
				Labels.getLabel("editarContaMDController.window.title"), args, MODAL);
	}

	private void refreshLista() {
		HibernateSearchObject<ContaMD> claseServicioBusqueda = new HibernateSearchObject<ContaMD>(ContaMD.class, pagingConta.getPageSize());

		claseServicioBusqueda.addFilterLike("nome", "%" + txtNombre.getText().trim().concat("%"));
		claseServicioBusqueda.addFilterEqual("activo", Boolean.TRUE);
		claseServicioBusqueda.addSortAsc("codigoContabil");

		plwConta.init(claseServicioBusqueda, contaList, pagingConta);
	}

}
